@ECHO OFF
SETLOCAL EnableExtensions EnableDelayedExpansion

REM Defines the default executable to use
SET EXE=..\zdoom.exe

REM If your zdoom.ini file is in a location other than the default, put the path to your ZDoom .ini file here:
SET ZDoomINI="..\zdoom-%username%.ini"

REM This defines the IWAD to use if a PWAD is loaded
SET IWAD="..\doom2.wad"

SET GAME=Default
SET FILES=.\Wolf3D.pk7
SET INI=".\zdoom-%username%-Wolf3D.ini"
SET LOADIWAD=0

REM Change this to 'Yes' to run Powershell instead of just the command prompt.
REM This more accurately emulates the original launcher .exe but can increase load time if Powershell isn't yet loaded
SET USEPS=No

SET ARGS=

CALL :ParseLoop %*
CALL :CheckINI
CALL :Launch

GOTO :EOF

:ParseLoop
	IF "%~1"=="" (
		GOTO :EOF
	) ELSE IF /I "%~1"=="-exe" (
		REM If a -exe command line parameter was provided, change the executable
		SET EXE=..\%~2.exe
		SHIFT
	) ELSE IF /I "%~1"=="-game" (
		SET GAME=%~2
		IF /I "%~2"=="SOD" (
			CALL :SODMenu
			SET INI=%INI:Wolf3D=SoD%
		) ELSE IF /I "%~2"=="CONSOLIDATED" (
			SET FILES=.\Wolf3D_Common.pk7
			SET INI=%INI:Wolf3D=SoD%
		)
		SHIFT
	) ELSE (
		SET ARGS=%ARGS% %1
		REM If a -file command line parameter was provided, automatically load the IWAD as defined above
		REM IF /I "%~1"=="-file" 
		SET LOADIWAD=1
	)
	SHIFT
	GOTO :ParseLoop
GOTO :EOF

:CheckINI
	REM Check to see if the .ini file is already there
	IF /I "!GAME!" NEQ "Default" (
		IF NOT EXIST !INI! (
			IF EXIST !INI:SoD=Wolf3D! COPY !INI:SoD=Wolf3D! !INI!
		)
	)

	IF NOT EXIST !INI! (
		REM If not, do initial setup for first run
		COPY %ZDoomINI% "!INI!"
		SET ARGS=%ARGS% +exec Wolf3D.cfg
	)
GOTO :EOF

:Launch
	REM Keep the default ceilings if an IWAD is loaded
	IF /I !LOADIWAD! GTR 0 SET ARGS=-file %IWAD% %ARGS% +set g_defaultceilings 1

	REM Change to the current directory so that you can drag and drop from other folders and the paths will still work
	CD "%~dp0"

	REM Automatically load the High-Res pack if it's there
	IF EXIST Wolf3D_HighRes.pk7 SET ARGS=%ARGS% Wolf3D_HighRes.pk7

	START %EXE% %Files% -config !INI! !ARGS!
GOTO :EOF

:SODMenu
	CLS

	If '%USEPS%' NEQ 'Yes' (
		CALL :StandardPrompt
	) ELSE (
		REM Check Windows Version
		FOR /f "tokens=4 delims=[.] " %%a IN ('ver') DO (SET WinVer=%%a)

		IF !WinVer! LSS 6 (
			CALL :StandardPrompt 
		) ELSE (
			CALL :PowerShellScript
		)
	)

	CALL :SelectionPrompt
GOTO :EOF

:PowerShellScript
	REM Run a PowerShell script for more accurate color emulation if Vista or higher...
	 >"Wolf3D SoD.ps1" ECHO $Host.UI.RawUI.BackgroundColor = "DarkBlue"
	>>"Wolf3D SoD.ps1" ECHO $Host.UI.RawUI.ForegroundColor = "Cyan"
	>>"Wolf3D SoD.ps1" ECHO Clear-Host
	>>"Wolf3D SoD.ps1" ECHO $c = [char]0x2592
	>>"Wolf3D SoD.ps1" ECHO $margin = ' ' * 16
	>>"Wolf3D SoD.ps1" ECHO $bar = $margin + $c.ToString^(^) * 48
	>>"Wolf3D SoD.ps1" ECHO $gap = $margin + $c + ' ' * 46 + $c
	>>"Wolf3D SoD.ps1" ECHO $lead = $margin + $c + ' ' * 4
	>>"Wolf3D SoD.ps1" ECHO $tail = "$c"
	>>"Wolf3D SoD.ps1" ECHO Write-Host "`n`n`n"
	>>"Wolf3D SoD.ps1" ECHO Write-Host $bar
	>>"Wolf3D SoD.ps1" ECHO Write-Host $gap
	>>"Wolf3D SoD.ps1" ECHO Write-Host $gap
	>>"Wolf3D SoD.ps1" ECHO Write-Host $lead -NoNewline
	>>"Wolf3D SoD.ps1" ECHO Write-Host -ForegroundColor White "1 - Spear of Destiny `(original mission`)   " -NoNewline
	>>"Wolf3D SoD.ps1" ECHO Write-Host $tail
	>>"Wolf3D SoD.ps1" ECHO Write-Host $gap
	>>"Wolf3D SoD.ps1" ECHO Write-Host $lead -NoNewline
	>>"Wolf3D SoD.ps1" ECHO Write-Host -ForegroundColor White "2 - Mission 2: Return to Danger           " -NoNewline
	>>"Wolf3D SoD.ps1" ECHO Write-Host $tail
	>>"Wolf3D SoD.ps1" ECHO Write-Host $gap
	>>"Wolf3D SoD.ps1" ECHO Write-Host $lead -NoNewline
	>>"Wolf3D SoD.ps1" ECHO Write-Host -ForegroundColor White "3 - Mission 3: Ultimate Challenge         " -NoNewline
	>>"Wolf3D SoD.ps1" ECHO Write-Host $tail
	>>"Wolf3D SoD.ps1" ECHO Write-Host $gap
	>>"Wolf3D SoD.ps1" ECHO Write-Host $gap
	>>"Wolf3D SoD.ps1" ECHO Write-Host $gap
	>>"Wolf3D SoD.ps1" ECHO Write-Host $lead -NoNewline
	>>"Wolf3D SoD.ps1" ECHO Write-Host -ForegroundColor White "0 - Exit to DOS                           "  -NoNewline
	>>"Wolf3D SoD.ps1" ECHO Write-Host $tail
	>>"Wolf3D SoD.ps1" ECHO Write-Host $gap
	>>"Wolf3D SoD.ps1" ECHO Write-Host $bar
	>>"Wolf3D SoD.ps1" ECHO Write-Host "`n`n`n`n`n"
	POWERSHELL -File "Wolf3D SoD.ps1"
	DEL "Wolf3D SoD.ps1" /Q
GOTO :EOF

:StandardPrompt
	REM Otherwise just run the .bat with the colors wrong.
	COLOR 1B
	ECHO.
	ECHO.
	ECHO.
	ECHO                 
	ECHO                                                               
	ECHO                                                               
	ECHO                     1 - Spear of Destiny ^(original mission^)   
	ECHO                                                               
	ECHO                     2 - Mission 2: Return to Danger           
	ECHO                                                               
	ECHO                     3 - Mission 3: Ultimate Challenge         
	ECHO                                                               
	ECHO                                                               
	ECHO                                                               
	ECHO                     0 - Exit to DOS                           
	ECHO                                                               
	ECHO                                                               
	ECHO                 
	ECHO.
	ECHO.
	ECHO.
	ECHO.
	ECHO.
GOTO :EOF

:SelectionPrompt
	CHOICE /C 1230 /N

	SET FILES=

	IF ERRORLEVEL 1 SET Files=.\SoD.pk7
	IF ERRORLEVEL 2 SET Files=.\SoD_RTD.pk7
	IF ERRORLEVEL 3 SET Files=.\SoD_TUC.pk7
	IF ERRORLEVEL 4 EXIT
Goto :EOF